@@ -361,7 +361,9 @@ func (mr *RunCommand) reloadConfig() error {
 	config := mr.getConfig()
 	mr.healthHelper.healthy = nil
 	mr.log().Println("Configuration loaded")
-	mr.log().Debugln(helpers.ToYAML(config))
+	if c, err := config.Masked(); err == nil {
+		mr.log().Debugln(helpers.ToYAML(c))
+	}
 
 	// initialize sentry
 	slh := sentry.LogHook{}
@@ -851,7 +853,7 @@ func (mr *RunCommand) processRunner(id int, runner *common.RunnerConfig, runners
 	mr.log().WithField("runner", runner.ShortDescription()).Debug("Acquiring request slot")
 	if !mr.buildsHelper.acquireRequest(runner) {
 		mr.log().WithField("runner", runner.ShortDescription()).
-			Debugln("Failed to request job: runner requestConcurrency meet")
+			Debugln("Failed to request job: 'request_concurrency' already reached, see https://docs.gitlab.com/runner/configuration/advanced-configuration.html#the-runners-section")
 		return nil
 	}
 
@@ -859,7 +861,7 @@ func (mr *RunCommand) processRunner(id int, runner *common.RunnerConfig, runners
 	executorData, err := provider.Acquire(runner)
 	if err != nil {
 		// Release job request
-		mr.buildsHelper.releaseRequest(runner)
+		mr.buildsHelper.releaseRequest(runner, false)
 
 		return fmt.Errorf("failed to update executor: %w", err)
 	}
@@ -877,14 +879,14 @@ func (mr *RunCommand) processBuildOnRunner(
 	buildSession, sessionInfo, err := mr.createSession(provider)
 	if err != nil {
 		// Release job request
-		mr.buildsHelper.releaseRequest(runner)
+		mr.buildsHelper.releaseRequest(runner, false)
 		return err
 	}
 
 	// Receive a new build
 	trace, jobData, err := mr.requestJob(runner, sessionInfo)
 	// Release job request
-	mr.buildsHelper.releaseRequest(runner)
+	mr.buildsHelper.releaseRequest(runner, jobData != nil)
 	if err != nil || jobData == nil {
 		return err
 	}
@@ -923,9 +925,11 @@ func (mr *RunCommand) processBuildOnRunner(
 				},
 				Job: usage_log.Job{
 					URL:             build.JobURL(),
-					DurationSeconds: build.Duration().Seconds(),
+					DurationSeconds: build.FinalDuration().Seconds(),
 					Status:          build.CurrentState().String(),
 					FailureReason:   build.FailureReason().String(),
+					StartedAt:       build.StartedAt().UTC(),
+					FinishedAt:      build.FinishedAt().UTC(),
 				},
 			})
 		}
@@ -1040,8 +1044,8 @@ func (mr *RunCommand) requestJob(
 		_, _ = trace.Write([]byte(err.Error() + "\n"))
 
 		err = trace.Fail(err, common.JobFailureData{
-			Reason:   common.RunnerSystemFailure,
-			ExitCode: common.ExitCodeUnsupportedOptions,
+			Reason:   common.ConfigurationError,
+			ExitCode: common.ExitCodeInvalidConfiguration,
 		})
 		logTerminationError(mr.log(), "Fail", err)
 
@@ -1108,7 +1112,10 @@ func (mr *RunCommand) updateWorkers(workerIndex *int, startWorker chan int, stop
 	concurrentLimit := config.Concurrent
 
 	if concurrentLimit < 1 {
-		mr.log().Fatalln("Concurrent is less than 1 - no jobs will be processed")
+		mr.log().Fatalln(fmt.Printf(
+			"Current configuration 'concurrent = %d' means that no jobs will be processed, see https://docs.gitlab.com/runner/configuration/advanced-configuration.html#the-global-section",
+			concurrentLimit,
+		))
 	}
 
 	for mr.currentWorkers > concurrentLimit {
