@@ -485,18 +485,40 @@ func Test_Image_ExecutorOptions_UnmarshalJSON(t *testing.T) {
 				assert.Equal(t, "", i.ExecutorOptions.Docker.User)
 			},
 		},
-		"invalid executor": {
-			json:           `{"executor_opts":{"k8s": {}}}`,
-			expectedErrMsg: []string{`Unsupported "image" options [k8s] for "executor_opts"; supported options are [docker]`},
+		"kubernetes, empty": {
+			json: `{"executor_opts":{"kubernetes": {}}}`,
+			expected: func(t *testing.T, i Image) {
+				assert.Equal(t, "", i.ExecutorOptions.Kubernetes.User)
+			},
+		},
+		"kubernetes, all options": {
+			json: `{"executor_opts":{"kubernetes": {"user": "1000"}}}`,
+			expected: func(t *testing.T, i Image) {
+				assert.Equal(t, "1000", i.ExecutorOptions.Kubernetes.User)
+			},
+		},
+		"kubernetes, invalid options": {
+			json:           `{"executor_opts":{"kubernetes": {"foobar": 1234}}}`,
+			expectedErrMsg: []string{`Unsupported "image" options [foobar] for "kubernetes executor"; supported options are [user]`},
 			expected: func(t *testing.T, i Image) {
 				assert.Equal(t, "", i.ExecutorOptions.Docker.Platform)
 				assert.Equal(t, "", i.ExecutorOptions.Docker.User)
+				assert.Equal(t, "", i.ExecutorOptions.Kubernetes.User)
 			},
 		},
-		"invalid executor, valid executor, invalid option": {
+		"invalid executor": {
+			json:           `{"executor_opts":{"k8s": {}}}`,
+			expectedErrMsg: []string{`Unsupported "image" options [k8s] for "executor_opts"; supported options are [docker kubernetes]`},
+			expected: func(t *testing.T, i Image) {
+				assert.Equal(t, "", i.ExecutorOptions.Docker.Platform)
+				assert.Equal(t, "", i.ExecutorOptions.Docker.User)
+				assert.Equal(t, "", i.ExecutorOptions.Kubernetes.User)
+			},
+		},
+		"docker, invalid executor, valid executor, invalid option": {
 			json: `{"executor_opts":{"k8s": {}, "docker": {"platform": "amd64", "foobar": 1234}}}`,
 			expectedErrMsg: []string{
-				`Unsupported "image" options [k8s] for "executor_opts"; supported options are [docker]`,
+				`Unsupported "image" options [k8s] for "executor_opts"; supported options are [docker kubernetes]`,
 				`Unsupported "image" options [foobar] for "docker executor"; supported options are [platform user]`,
 			},
 			expected: func(t *testing.T, i Image) {
@@ -504,6 +526,16 @@ func Test_Image_ExecutorOptions_UnmarshalJSON(t *testing.T) {
 				assert.Equal(t, "", i.ExecutorOptions.Docker.User)
 			},
 		},
+		"kubernetes, invalid executor, valid executor, invalid option": {
+			json: `{"executor_opts":{"dockers": {}, "kubernetes": {"user": "1000", "foobar": 1234}}}`,
+			expectedErrMsg: []string{
+				`Unsupported "image" options [dockers] for "executor_opts"; supported options are [docker kubernetes]`,
+				`Unsupported "image" options [foobar] for "kubernetes executor"; supported options are [user]`,
+			},
+			expected: func(t *testing.T, i Image) {
+				assert.Equal(t, "1000", i.ExecutorOptions.Kubernetes.User)
+			},
+		},
 	}
 
 	for name, tt := range tests {
@@ -524,6 +556,64 @@ func Test_Image_ExecutorOptions_UnmarshalJSON(t *testing.T) {
 	}
 }
 
+func Test_Image_ExecutorOptions_GetUIDGID(t *testing.T) {
+	tests := map[string]struct {
+		kubernetesOptions func() *ImageKubernetesOptions
+		expectedError     bool
+		expectedUID       int64
+		expectedGID       int64
+	}{
+		"empty user": {
+			kubernetesOptions: func() *ImageKubernetesOptions {
+				return &ImageKubernetesOptions{
+					User: "",
+				}
+			},
+		},
+		"only user": {
+			kubernetesOptions: func() *ImageKubernetesOptions {
+				return &ImageKubernetesOptions{
+					User: "1000",
+				}
+			},
+			expectedUID: int64(1000),
+		},
+		"uid and gid": {
+			kubernetesOptions: func() *ImageKubernetesOptions {
+				return &ImageKubernetesOptions{
+					User: "1000:1000",
+				}
+			},
+			expectedUID: int64(1000),
+			expectedGID: int64(1000),
+		},
+		"invalid user": {
+			kubernetesOptions: func() *ImageKubernetesOptions {
+				return &ImageKubernetesOptions{
+					User: "gitlab-runner",
+				}
+			},
+			expectedError: true,
+		},
+	}
+
+	for name, tt := range tests {
+		t.Run(name, func(t *testing.T) {
+			uid, gid, err := tt.kubernetesOptions().GetUIDGID()
+			if tt.expectedError {
+				require.Error(t, err)
+				require.Equal(t, int64(0), uid)
+				require.Equal(t, int64(0), gid)
+				return
+			}
+
+			require.NoError(t, err)
+			require.Equal(t, tt.expectedUID, uid)
+			require.Equal(t, tt.expectedGID, gid)
+		})
+	}
+}
+
 func TestJobResponse_Run(t *testing.T) {
 	tests := map[string]struct {
 		json            string
