@@ -2174,8 +2174,10 @@ const (
 
 func TestCredSetup(t *testing.T) {
 	const (
-		markerForBuild  = "#build# "
-		markerForHelper = "#helper# "
+		markerForBuild      = "#build# "
+		markerForHelper     = "#helper# "
+		markerPreGetSource  = "pre_get_source"
+		markerPostGetSource = "post_get_source"
 	)
 
 	listGitConfig := func(t *testing.T, shell, prefix string) string {
@@ -2189,6 +2191,42 @@ func TestCredSetup(t *testing.T) {
 		}
 		return ""
 	}
+	getGitCred := func(t *testing.T, shell, prefix string) string {
+		const username = "gitlab-ci-token"
+
+		switch shell {
+		case shells.Bash:
+			return fmt.Sprintf(`echo -e "protocol=https\nhost=gitlab.com\nusername=%s" | git -c credential.interactive=never credential fill | sed 's/^/#%s# /g';`, username, prefix)
+		case shells.SNPwsh, shells.SNPowershell:
+			return fmt.Sprintf("$GitStdin = \"protocol=https`nhost=gitlab.com`nusername=%s\"; "+
+				"$GitStdinFile = Join-Path ${CI_BUILDS_DIR} 'git_get_cred_stdin.txt' ; "+
+				"$GitStdoutFile = Join-Path ${CI_BUILDS_DIR} 'git_get_cred_stdout.txt' ; "+
+				"If(Test-Path $GitStdoutFile) { Remove-Item $GitStdoutFile } ; "+
+				"[System.IO.File]::WriteAllText($GitStdinFile, $GitStdin) ; "+
+				"Start-Process -FilePath 'git' -ArgumentList 'credential','fill' -RedirectStandardInput $GitStdinFile -RedirectStandardOutput $GitStdoutFile -NoNewWindow -Wait; "+
+				"(Get-Content -Path $GitStdoutFile) -replace '^','#%s# ';", username, prefix)
+		default:
+			t.Fatalf("shell %s not supported", shell)
+		}
+		return ""
+	}
+	setGitCred := func(t *testing.T, shell string) string {
+		const username = "gitlab-ci-token"
+		const password = "fake_password"
+
+		switch shell {
+		case shells.Bash:
+			return fmt.Sprintf(`echo -e "protocol=https\nhost=gitlab.com\nusername=%s\npassword=%s" | git credential approve;`, username, password)
+		case shells.SNPwsh, shells.SNPowershell:
+			return fmt.Sprintf("$GitStdin = \"protocol=https`nhost=gitlab.com`nusername=%s`npassword=%s\"; "+
+				"$GitStdinFile = Join-Path ${CI_BUILDS_DIR} 'git_set_cred_stdin.txt' ; "+
+				"[System.IO.File]::WriteAllText($GitStdinFile, $GitStdin) ; "+
+				"Start-Process -FilePath 'git' -ArgumentList 'credential','approve' -RedirectStandardInput $GitStdinFile -NoNewWindow -Wait;", username, password)
+		default:
+			t.Fatalf("shell %s not supported", shell)
+		}
+		return ""
+	}
 	extractGitConfig := func(blob, prefix string) string {
 		out := []string{}
 		for _, line := range strings.Split(blob, "\n") {
@@ -2201,11 +2239,11 @@ func TestCredSetup(t *testing.T) {
 
 	tests := []struct {
 		gitUrlsWithoutTokens bool
-		validator            func(t *testing.T, out string, remoteURL string)
+		validator            func(t *testing.T, out string, remoteURL string, cachedGitCreds string, token string)
 	}{
 		{
 			gitUrlsWithoutTokens: true,
-			validator: func(t *testing.T, out string, remoteURL string) {
+			validator: func(t *testing.T, out string, remoteURL string, cachedGitCreds string, token string) {
 				assert.NotContains(t, remoteURL, "@", "the remote URL should not contain any auth data")
 				assert.NotContains(t, remoteURL, "gitlab-ci-token", "the remote URL should not contain the token user")
 
@@ -2218,11 +2256,19 @@ func TestCredSetup(t *testing.T) {
 					assert.Contains(t, gitConfig, "credential."+remoteHost+".helper=!", "should contain a credential helper")
 					assert.NotContains(t, gitConfig, "remote.origin.url=https://gitlab-ci-token:", "the origin URL should not contain any auth data")
 				}
+
+				// pre get-source: gitlab-ci-token password comes from the 1st global credential helper
+				// post get-source: globally defined credential.helpers should not be run, we can fetch the current token with the local cred helper, but it is not cached
+				assert.Contains(t, out, "#pre_get_source# password=fake_password", "before getting sources (& setting up the local cred helper), previously used creds are used")
+				assert.Contains(t, out, "#post_get_source# password=[MASKED]", "after getting sources (& setting up the local cred helper), only its creds are used")
+				content, err := os.ReadFile(cachedGitCreds)
+				require.NoError(t, err)
+				assert.Contains(t, "https://gitlab-ci-token:fake_password@gitlab.com\n", string(content), "per-build cred helper caches creds from before getting source (& setting up the local cred helper)")
 			},
 		},
 		{
 			gitUrlsWithoutTokens: false,
-			validator: func(t *testing.T, out string, remoteURL string) {
+			validator: func(t *testing.T, out string, remoteURL string, cachedGitCreds string, token string) {
 				assert.Contains(t, remoteURL, "@", "the remote URL should contain auth data")
 				assert.Contains(t, remoteURL, "gitlab-ci-token", "remote URL should contain the token user")
 
@@ -2235,68 +2281,102 @@ func TestCredSetup(t *testing.T) {
 					assert.NotContains(t, gitConfig, "credential."+remoteHost+".helper=!", "should not contain a credential helper")
 					assert.Contains(t, gitConfig, "remote.origin.url=https://gitlab-ci-token:", "should contain the origin URL including auth data")
 				}
+
+				// pre get-source: gitlab-ci-token password comes from the 1st global credential helper
+				// post get-source: we can fetch creds from the global helpers, because we didn't set the local empty cred helper: feature flag is OFF
+				// the CI_JOB_TOKEN is expected to be cached
+				assert.Contains(t, out, "#pre_get_source# password=fake_password", "before getting sources, previously used creds are used")
+				assert.Contains(t, out, "#post_get_source# password=[MASKED]", "after getting sources, the token is cached")
+				content, err := os.ReadFile(cachedGitCreds)
+				require.NoError(t, err)
+				assert.Contains(t, fmt.Sprintf("https://gitlab-ci-token:%s@gitlab.com\n", token), string(content), "per-build cred helper caches the most recently used creds")
 			},
 		},
 	}
 
-	setupCachingCredHelper(t)
+	setupCachingCredHelpers(t)
 	orgURL, err := url.Parse(repoURLWithSubmodules)
 	require.NoError(t, err, "parsing original repo url")
-	setInvalidGitCreds(t, orgURL)
+	setInvalidGitCreds(t, *orgURL)
+
+	gitStrategies := map[string]struct {
+		featureFlags map[string]bool
+		jobVariables common.JobVariables
+	}{
+		"fetch": {},
+		"clone": {
+			jobVariables: common.JobVariables{{Key: "GIT_STRATEGY", Value: "clone"}},
+		},
+		"nativeClone": {
+			featureFlags: map[string]bool{featureflags.UseGitNativeClone: true},
+			jobVariables: common.JobVariables{{Key: "GIT_STRATEGY", Value: "clone"}},
+		},
+	}
 
 	for _, test := range tests {
-		name := fmt.Sprintf("gitUrlsWithoutTokens:%t", test.gitUrlsWithoutTokens)
-		t.Run(name, func(t *testing.T) {
+		t.Run(fmt.Sprintf("%s:%t", featureflags.GitURLsWithoutTokens, test.gitUrlsWithoutTokens), func(t *testing.T) {
 			t.Parallel()
 
-			shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-				t.Parallel()
+			for gitStrategyName, gitStrategy := range gitStrategies {
+				t.Run("GIT_STRATEGY:"+gitStrategyName, func(t *testing.T) {
+					t.Parallel()
 
-				if runtime.GOOS != shells.OSWindows && shell == shells.SNPowershell {
-					t.Skip("powershell is not supported on non-windows platforms")
-				}
+					shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+						t.Parallel()
 
-				jobResponse, err := common.GetRemoteBuildResponse(listGitConfig(t, shell, markerForBuild))
-				require.NoError(t, err)
+						helpers.SkipIntegrationTests(t, shell)
 
-				jobResponse.GitInfo.RepoURL = repoURLWithSubmodules
-				jobResponse.GitInfo.Sha = repoShaWithSubmodules
-				token, repoURLWithToken := injectJobToken(t, &jobResponse)
+						jobResponse, err := common.GetRemoteBuildResponse(listGitConfig(t, shell, markerForBuild))
+						require.NoError(t, err)
 
-				jobResponse.Hooks = append(jobResponse.Hooks, common.Hook{
-					Name:   common.HookPostGetSourcesScript,
-					Script: common.StepScript{listGitConfig(t, shell, markerForHelper)},
+						jobResponse.GitInfo.RepoURL = repoURLWithSubmodules
+						jobResponse.GitInfo.Sha = repoShaWithSubmodules
+						token, _ := buildtest.InjectJobTokenFromEnv(t, &jobResponse)
+
+						jobResponse.Hooks = append(jobResponse.Hooks, common.Hook{
+							Name:   common.HookPreGetSourcesScript,
+							Script: common.StepScript{setGitCred(t, shell), getGitCred(t, shell, markerPreGetSource)},
+						})
+
+						jobResponse.Hooks = append(jobResponse.Hooks, common.Hook{
+							Name:   common.HookPostGetSourcesScript,
+							Script: common.StepScript{listGitConfig(t, shell, markerForHelper), getGitCred(t, shell, markerPostGetSource)},
+						})
+
+						jobResponse.Variables = append(jobResponse.Variables,
+							common.JobVariable{Key: "GIT_TRACE", Value: "1"},
+							common.JobVariable{Key: "GIT_CURL_VERBOSE", Value: "1"},
+							common.JobVariable{Key: "GIT_TRANSFER_TRACE", Value: "1"},
+							common.JobVariable{Key: "CI_DEBUG_TRACE", Value: "1"},
+							common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"},
+							common.JobVariable{Key: "GIT_SUBMODULE_FORCE_HTTPS", Value: "1"},
+							common.JobVariable{Key: "CI_SERVER_HOST", Value: "gitlab.com"},
+						)
+						jobResponse.Variables = append(jobResponse.Variables, gitStrategy.jobVariables...)
+
+						build := newBuild(t, jobResponse, shell)
+
+						build.Runner.RunnerCredentials.URL = "https://gitlab.com/"
+
+						buildtest.SetBuildFeatureFlag(build, featureflags.GitURLsWithoutTokens, test.gitUrlsWithoutTokens)
+						for k, v := range gitStrategy.featureFlags {
+							buildtest.SetBuildFeatureFlag(build, k, v)
+						}
+
+						out, err := buildtest.RunBuildReturningOutput(t, build)
+						assert.NoError(t, err)
+
+						assert.NotContains(t, out, token, "should not contain the token")
+
+						remoteURL, err := build.GetRemoteURL()
+						assert.NoError(t, err, "getting build's remote URL")
+
+						// cached creds from the 1st helper created in setupTestCredHelpers
+						cachedCreds := filepath.Join(build.Runner.BuildsDir, "git-credentials")
+						test.validator(t, out, remoteURL, cachedCreds, token)
+					})
 				})
-
-				jobResponse.Variables = append(jobResponse.Variables,
-					common.JobVariable{Key: "GIT_TRACE", Value: "1"},
-					common.JobVariable{Key: "GIT_CURL_VERBOSE", Value: "1"},
-					common.JobVariable{Key: "GIT_TRANSFER_TRACE", Value: "1"},
-					common.JobVariable{Key: "CI_DEBUG_TRACE", Value: "1"},
-					common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"},
-					common.JobVariable{Key: "GIT_SUBMODULE_FORCE_HTTPS", Value: "1"},
-					common.JobVariable{Key: "CI_SERVER_HOST", Value: "gitlab.com"},
-				)
-
-				build := newBuild(t, jobResponse, shell)
-
-				build.Runner.RunnerCredentials.URL = "https://gitlab.com/"
-
-				buildtest.SetBuildFeatureFlag(build, featureflags.GitURLsWithoutTokens, test.gitUrlsWithoutTokens)
-
-				// inject invalid creds into the git cred helper to ensure we try to prune cached creds.
-				setInvalidGitCreds(t, repoURLWithToken)
-
-				out, err := buildtest.RunBuildReturningOutput(t, build)
-				assert.NoError(t, err)
-
-				assert.NotContains(t, out, token, "should not contain the token")
-
-				remoteURL, err := build.GetRemoteURL()
-				assert.NoError(t, err, "getting build's remote URL")
-
-				test.validator(t, out, remoteURL)
-			})
+			}
 		})
 	}
 }
@@ -2354,7 +2434,7 @@ func TestSubmoduleAutoBump(t *testing.T) {
 
 				jobResponse.GitInfo.RepoURL = repoURL
 				jobResponse.GitInfo.Sha = repoSha
-				injectJobToken(t, &jobResponse)
+				buildtest.InjectJobTokenFromEnv(t, &jobResponse)
 
 				jobResponse.Variables = append(jobResponse.Variables,
 					common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"},
@@ -2437,7 +2517,7 @@ func TestBuildWithCleanGitConfig(t *testing.T) {
 		)
 		jobResponse.GitInfo.RepoURL = repoURLWithSubmodules
 		jobResponse.GitInfo.Sha = repoShaWithSubmodules
-		injectJobToken(t, &jobResponse)
+		buildtest.InjectJobTokenFromEnv(t, &jobResponse)
 
 		build := newBuild(t, jobResponse, shell)
 		build.Runner.RunnerCredentials.URL = "https://gitlab.com/"
@@ -2454,32 +2534,12 @@ func TestBuildWithCleanGitConfig(t *testing.T) {
 	})
 }
 
-// injectJobToken injects a job token into an existing jobResponse by
-// - setting the jobResponse's token
-// - updating the jobResponse's gitInfo with an URL with the token
-// - injecting a CI_JOB_TOKEN jobVariable
-// It returns the token and the new gitInfo that were injected.
-func injectJobToken(t *testing.T, jobResponse *common.JobResponse) (string, *url.URL) {
-	token := getTokenFromEnv(t)
-
-	repoURLWithToken := func(orgRepoURL, token string) *url.URL {
-		u, err := url.Parse(orgRepoURL)
-		require.NoError(t, err, "parsing original repo URL")
-		u.User = url.UserPassword("gitlab-ci-token", token)
-		return u
-	}(jobResponse.GitInfo.RepoURL, token)
-
-	jobResponse.Variables = append(jobResponse.Variables, common.JobVariable{
-		Key: "CI_JOB_TOKEN", Value: token, Masked: true,
-	})
-	jobResponse.Token = token
-	jobResponse.GitInfo.RepoURL = repoURLWithToken.String()
-
-	return token, repoURLWithToken
-}
-
-// setupCachingCredHelper sets up a (global) caching git credential helper
-func setupCachingCredHelper(t *testing.T) {
+// setupCachingCredHelpers sets up a (global) git cred helpers
+//   - the 1st one uses `git-credential-store` to create a file in the build directory
+//   - the 2nd one uses `git-credential-store` with a temporary file
+//
+// After the tests are done, all original cred helpers are restored
+func setupCachingCredHelpers(t *testing.T) {
 	gitCredCache, err := os.CreateTemp("", "")
 	require.NoError(t, err, "creating temp file for cred cache")
 	require.NoError(t, gitCredCache.Close(), "closing the temp file for cred cache")
@@ -2489,41 +2549,62 @@ func setupCachingCredHelper(t *testing.T) {
 	})
 
 	// ignoring error, because unset configs would produce an error too
-	orgCredHelper, _ := exec.Command("git", "config", "--global", "credential.helper").Output()
+	orgCredHelper, _ := exec.Command("git", "config", "--global", "--get-all", "credential.helper").Output()
 	orgCredHelper = bytes.Trim(orgCredHelper, "\n\r")
 
-	err = exec.Command("git", "config", "--global", "credential.helper", "store --file="+gitCredCache.Name()).Run()
+	err = exec.Command("git", "config", "--global", "--replace-all", "credential.helper", "").Run()
+	require.NoError(t, err, "adding empty cred helper")
+
+	// global credential helper with the cache in the build dir, thus caches are separate per test
+	helper := `` +
+		`f(){ ` +
+		`  bd=$(echo "$CI_BUILDS_DIR" | sed "s/\\\/\//g"); ` +
+		`  if [ -z "${bd}" ] || [ ! -d "${bd}" ]; then exit 1; fi; ` +
+		`  git credential-store --file="${bd}/git-credentials" "$1" ;` +
+		`}; f`
+	err = exec.Command("git", "config", "--global", "--add", "credential.helper", "!"+helper).Run()
+	require.NoError(t, err, "adding storing cred helper per build")
+
+	// global credential helper which uses the same cache for all tests
+	helper = "store --file=" + helpers.ToSlash(gitCredCache.Name())
+	err = exec.Command("git", "config", "--global", "--add", "credential.helper", helper).Run()
 	require.NoError(t, err, "adding caching cred helper")
+
 	t.Cleanup(func() {
-		if len(orgCredHelper) > 0 {
-			err := exec.Command("git", "config", "--global", "credential.helper", string(orgCredHelper)).Run()
-			require.NoError(t, err, "setting cred helper back to %s", orgCredHelper)
-		} else {
-			err := exec.Command("git", "config", "--global", "--unset", "credential.helper").Run()
-			require.NoError(t, err, "unsetting cred helper")
+		err := exec.Command("git", "config", "--global", "--unset-all", "credential.helper").Run()
+		require.NoError(t, err, "unsetting cred helper")
+		for _, helper := range bytes.Split(orgCredHelper, []byte{'\n'}) {
+			if len(helper) > 0 {
+				err := exec.Command("git", "config", "--global", "--add", "credential.helper", string(helper)).Run()
+				require.NoError(t, err, "restoring credential.helper: %s", helper)
+			}
 		}
 	})
 }
 
 // setInvalidGitCreds injects invalid creds into git cred helpers
-func setInvalidGitCreds(t *testing.T, orgURL *url.URL) {
-	urlWithFakeCreds := *orgURL
-	urlWithFakeCreds.User = url.UserPassword("some-user", "some-token")
+func setInvalidGitCreds(t *testing.T, orgURL url.URL) {
+	creds := map[string]string{
+		"some-user":       "some-token",
+		"gitlab-ci-token": "old_expired_token",
+	}
 
-	// save the fake / invalid creds
-	cmd := exec.Command("git", "credential", "approve")
-	cmd.Stdin = strings.NewReader(fmt.Sprintf("url=%s\n", urlWithFakeCreds.String()))
-	err := cmd.Run()
-	require.NoError(t, err, "setting up fake creds")
+	for username, password := range creds {
+		// save the fake / invalid creds
+		cmd := exec.Command("git", "credential", "approve")
+		cmd.Stdin = strings.NewReader(fmt.Sprintf("protocol=%s\nhost=%s\nusername=%s\npassword=%s\n", orgURL.Scheme, orgURL.Host, username, password))
+		err := cmd.Run()
+		require.NoError(t, err, "setting up fake creds")
 
-	// ensure the cred helper is set up and returns the fake creds
-	cmd = exec.Command("git", "credential", "fill")
-	cmd.Stdin = strings.NewReader(fmt.Sprintf("protocol=%s\nhost=%s\n", urlWithFakeCreds.Scheme, urlWithFakeCreds.Host))
-	var out []byte
-	out, err = cmd.Output()
-	require.NoError(t, err, "getting cached git creds")
-	require.Contains(t, string(out), "username=some-user", "wrong user for cached cred")
-	require.Contains(t, string(out), "password=some-token", "wrong password for cached cred")
+		// ensure the cred helper is set up and returns the fake creds
+		cmd = exec.Command("git", "credential", "fill")
+		cmd.Stdin = strings.NewReader(fmt.Sprintf("protocol=%s\nhost=%s\nusername=%s\n", orgURL.Scheme, orgURL.Host, username))
+		var out []byte
+		out, err = cmd.Output()
+		require.NoError(t, err, "getting cached git creds")
+		require.Contains(t, string(out), fmt.Sprintf("username=%s", username), "wrong user for cached cred")
+		require.Contains(t, string(out), fmt.Sprintf("password=%s", password), "wrong password for cached cred")
+	}
 }
 
 func onlyHost(t *testing.T, remoteURL string) string {
@@ -2534,16 +2615,3 @@ func onlyHost(t *testing.T, remoteURL string) string {
 
 	return url_helpers.OnlySchemeAndHost(u).String()
 }
-
-var tokenEnvVars = []string{"GITLAB_TOKEN", "CI_JOB_TOKEN", "OUTER_CI_JOB_TOKEN"}
-
-func getTokenFromEnv(t *testing.T) string {
-	for _, envVar := range tokenEnvVars {
-		if token, ok := os.LookupEnv(envVar); ok {
-			t.Log("using token from env var", envVar)
-			return token
-		}
-	}
-	require.Fail(t, "no token available", "considered env vars: %q", tokenEnvVars)
-	return ""
-}
