@@ -21,17 +21,21 @@ import (
 
 const (
 	// When umask is disabled for the Kubernetes executor,
-	// a hidden file, .giltab-build-uid-gid, is created in the `builds_dir` directory to assist the helper container
+	// a hidden file, .gitlab-build-uid-gid, is created in the `builds_dir` directory to assist the helper container
 	// in retrieving the build image's configured `uid:gid`.
 	// This information is then applied to the working directories to prevent them from being writable by anyone.
-	BuildUiGidFile   = ".giltab-build-uid-gid"
-	StartupProbeFile = ".gitlab-startup-marker"
-
+	BuildUidGidFile        = ".gitlab-build-uid-gid"
+	StartupProbeFile       = ".gitlab-startup-marker"
 	gitlabEnvFileName      = "gitlab_runner_env"
 	gitlabCacheEnvFileName = "gitlab_runner_cache_env"
-	credHelperConfFile     = "cred-helper.conf"
-	gitDir                 = ".git"
-	gitTemplateDir         = "git-template"
+	// credHelperConfFile is the base path used for the cred helper config file within the jobs temp directory
+	credHelperConfFile = "cred-helper.conf"
+	// credHelperCommand is the command to use as a git credential helper to pull credentials from the environment.
+	// git always comes (or depends) on a POSIX shell, so any helper can rely on that, regardless of the OS, git distribution, ...
+	credHelperCommand         = `!f(){ if [ "$1" = "get" ] ; then echo "password=${CI_JOB_TOKEN}" ; fi ; } ; f`
+	gitDir                    = ".git"
+	gitTemplateDir            = "git-template"
+	gitMinVersionCloneWithRef = "2.49"
 )
 
 var errUnknownGitStrategy = errors.New("unknown GIT_STRATEGY")
@@ -154,10 +158,17 @@ func (b *AbstractShell) cacheExtractor(ctx context.Context, w ShellWriter, info
 		return common.ErrSkipBuildStage
 	}
 
-	if info.Build.IsFeatureFlagOn(featureflags.DisableUmaskForKubernetesExecutor) &&
-		info.Build.Runner.Shell == Bash {
-		b.changeFilesOwnership(w, info.Build, info.Build.CacheDir)
-	}
+	// Caches and artifacts are managed in the helper container, thus all files
+	// would be owned by the user of the helper container.
+	// We want all the files to be owned by the user of the build container. Thus we
+	// change the ownership in the helper container (to the uid/gid we discovered via
+	// the `init-build-uid-gid-collector` container) before the build in the build
+	// container runs.
+	// We change the directories
+	// - project root dir, after extracting the cache/artifact files
+	// - the cache dir
+	// so that all dirs/files eventually have the correct ownership.
+	b.changeFilesOwnership(w, info, info.Build.CacheDir, info.Build.RootDir)
 
 	return nil
 }
@@ -313,10 +324,17 @@ func (b *AbstractShell) downloadAllArtifacts(w ShellWriter, info common.ShellScr
 		}
 	})
 
-	if info.Build.IsFeatureFlagOn(featureflags.DisableUmaskForKubernetesExecutor) &&
-		info.Build.Runner.Shell == Bash {
-		b.changeFilesOwnership(w, info.Build, info.Build.CacheDir)
-	}
+	// Caches and artifacts are managed in the helper container, thus all files
+	// would be owned by the user of the helper container.
+	// We want all the files to be owned by the user of the build container. Thus we
+	// change the ownership in the helper container (to the uid/gid we discovered via
+	// the `init-build-uid-gid-collector` container) before the build in the build
+	// container runs.
+	// We change the directories
+	// - project root dir, after extracting the cache/artifact files
+	// - the cache dir
+	// so that all dirs/files eventually have the correct ownership.
+	b.changeFilesOwnership(w, info, info.Build.CacheDir, info.Build.RootDir)
 
 	return nil
 }
@@ -360,11 +378,6 @@ func (b *AbstractShell) writeGetSourcesScript(_ context.Context, w ShellWriter,
 		return err
 	}
 
-	if info.Build.IsFeatureFlagOn(featureflags.DisableUmaskForKubernetesExecutor) &&
-		info.Build.Runner.Shell == Bash {
-		b.changeFilesOwnership(w, info.Build, info.Build.FullProjectDir(), info.Build.TmpProjectDir())
-	}
-
 	b.guardGetSourcesScriptHooks(w, info, "post_clone_script", func() []string {
 		var s []string
 
@@ -380,6 +393,8 @@ func (b *AbstractShell) writeGetSourcesScript(_ context.Context, w ShellWriter,
 		return s
 	})
 
+	b.changeFilesOwnership(w, info, info.Build.RootDir)
+
 	return nil
 }
 
@@ -532,18 +547,39 @@ func (b *AbstractShell) writeCloneFetchCmds(w ShellWriter, info common.ShellScri
 	return nil
 }
 
-func (b *AbstractShell) changeFilesOwnership(w ShellWriter, build *common.Build, dir ...string) {
-	// ensure all parts are quoted with single quotes, so that whitespaces
-	// and all don't trip us up and to ensure no unwanted variable expansion is happening
-	unquotedUidGidFile := fmt.Sprintf(`%s/%s`, build.RootDir, BuildUiGidFile)
-	quotedUidGidFile := fmt.Sprintf(`'%s'`, unquotedUidGidFile)
-	quotedDirs := make([]string, len(dir))
-	for i, d := range dir {
-		quotedDirs[i] = fmt.Sprintf(`'%s'`, d)
+func (b *AbstractShell) changeFilesOwnership(w ShellWriter, info common.ShellScriptInfo, dir ...string) {
+	// The shell is not set the same way depending on the unit test
+	// Some unit tests use info->Build->Runner->Shell while other use info->Shell
+	shellName := info.Shell
+	if shellName == "" {
+		// GetDefaultShell will panic, if it can't find a default shell
+		shellName = info.Build.Runner.Shell
 	}
 
+	// umask 0000 disabling is only support for UNIX-Like shells
+	// We therefore don't do anything for PowerShell/pwsh
+	if slices.Contains([]string{SNPowershell, SNPwsh}, shellName) {
+		return
+	}
+
+	// ensure all parts are quoted with single quotes, so that whitespaces
+	// and all don't trip us up and to ensure no unwanted variable expansion is happening
+	unquotedUidGidFile := fmt.Sprintf(`%s/%s`, info.Build.RootDir, BuildUidGidFile)
+	quotedUidGidFile := fmt.Sprintf(`'%s'`, unquotedUidGidFile)
+
+	// unquotedUidGidFile file is only created when FF_DISABLE_UMASK_FOR_KUBERNETES_EXECUTOR is enabled
 	w.IfFile(unquotedUidGidFile) // IfFIle does use `%q` internally
-	w.Line(fmt.Sprintf(`chown -R "$(stat -c '%%u:%%g' %s)" %s`, quotedUidGidFile, strings.Join(quotedDirs, " ")))
+	for _, d := range dir {
+		if strings.TrimSpace(d) == "" {
+			continue
+		}
+		w.IfDirectory(d)
+		w.Line(fmt.Sprintf(`chown -R "$(stat -c '%%u:%%g' %s)" '%s'`, quotedUidGidFile, d))
+		if info.Build.IsDebugModeEnabled() {
+			w.Line(fmt.Sprintf(`echo "Setting ownership for %s to $(stat -c '%%u:%%g' %s)"`, d, quotedUidGidFile))
+		}
+		w.EndIf()
+	}
 	w.EndIf()
 }
 
@@ -551,12 +587,26 @@ func (b *AbstractShell) handleGetSourcesStrategy(w ShellWriter, info common.Shel
 	build := info.Build
 	projectDir := build.FullProjectDir()
 
-	switch build.GetGitStrategy() {
-	case common.GitFetch:
-		return b.writeRefspecFetchCmd(w, info)
-	case common.GitClone:
-		w.RmDir(projectDir)
-		return b.writeRefspecFetchCmd(w, info)
+	switch strategy := build.GetGitStrategy(); strategy {
+	case common.GitFetch, common.GitClone:
+		templateDir := b.setupTemplateDir(w, build, projectDir)
+
+		credConfigFile := b.credConfigFile(build, w)
+		if credConfigFile != "" {
+			remoteHost, err := b.getRemoteHost(build)
+			if err != nil {
+				return fmt.Errorf("getting remote host: %w", err)
+			}
+			w.RmFile(credConfigFile)
+			w.SetupGitCredHelper(credConfigFile, "credential."+remoteHost, "gitlab-ci-token")
+		}
+
+		getCmdWriter := b.writeRefspecFetchCmd
+		if strategy == common.GitClone {
+			getCmdWriter = b.writeCloneCmdIfPossible
+		}
+
+		return getCmdWriter(w, info, templateDir, credConfigFile)
 	case common.GitNone:
 		w.Noticef("Skipping Git repository setup")
 		w.MkDir(projectDir)
@@ -572,7 +622,7 @@ func (b *AbstractShell) handleGetSourcesStrategy(w ShellWriter, info common.Shel
 }
 
 //nolint:funlen
-func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScriptInfo) error {
+func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScriptInfo, templateDir, credConfigFile string) error {
 	build := info.Build
 	projectDir := build.FullProjectDir()
 	depth := build.GitInfo.Depth
@@ -583,33 +633,10 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScr
 		w.Noticef("Fetching changes...")
 	}
 
-	// initializing
-	templateDir := w.MkTmpDir(gitTemplateDir)
-	templateFile := w.Join(templateDir, "config")
 	objectFormat := build.GetRepositoryObjectFormat()
 
 	b.writeGitCleanup(w, build)
 
-	if build.SafeDirectoryCheckout {
-		// Solves problem with newer Git versions when files existing in the working directory
-		// are owned by different system owners. This may happen for example with Docker executor,
-		// a root-less image used in previous job and the working directory being persisted between
-		// jobs. More details can be found at https://gitlab.com/gitlab-org/gitlab/-/issues/368133.
-		w.Command("git", "config", "--global", "--add", "safe.directory", projectDir)
-	}
-
-	w.Command("git", "config", "-f", templateFile, "init.defaultBranch", "none")
-	w.Command("git", "config", "-f", templateFile, "fetch.recurseSubmodules", "false")
-	w.Command("git", "config", "-f", templateFile, "credential.interactive", "never")
-
-	if build.IsFeatureFlagOn(featureflags.UseGitBundleURIs) {
-		w.Command("git", "config", "-f", templateFile, "transfer.bundleURI", "true")
-	}
-
-	if build.IsSharedEnv() {
-		b.writeGitSSLConfig(w, build, []string{"-f", templateFile})
-	}
-
 	if objectFormat != common.DefaultObjectFormat {
 		w.Command("git", "init", projectDir, "--template", templateDir, "--object-format", objectFormat)
 	} else {
@@ -623,11 +650,8 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScr
 		return fmt.Errorf("writing fetch commands: %w", err)
 	}
 
-	if credConfigFile := b.credConfigFile(build, w); credConfigFile != "" {
-		err := b.configureGitCredHelper(w, info, credConfigFile)
-		if err != nil {
-			return fmt.Errorf("writing fetch commands: %w", err)
-		}
+	if credConfigFile != "" {
+		w.Command("git", "config", "include.path", credConfigFile)
 	}
 
 	// Add `git remote` or update existing
@@ -663,51 +687,111 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, info common.ShellScr
 	return nil
 }
 
-func (b *AbstractShell) configureGitCredHelper(w ShellWriter, info common.ShellScriptInfo, credConfigFile string) error {
-	w.RmFile(credConfigFile)
+func (b *AbstractShell) writeCloneCmdIfPossible(w ShellWriter, info common.ShellScriptInfo, templateDir, credConfigFile string) error {
 	build := info.Build
-	shellName := build.Runner.Shell
-	if shellName == "" {
-		// GetDefaultShell will panic, if it can't find a default shell
-		shellName = common.GetDefaultShell()
+	projectDir := build.FullProjectDir()
+
+	// always ensure the old clone is gone
+	w.RmDir(projectDir)
+
+	if !build.IsFeatureFlagOn(featureflags.UseGitNativeClone) {
+		return b.writeRefspecFetchCmd(w, info, templateDir, credConfigFile)
 	}
 
-	shell := common.GetShell(shellName)
-	if shell == nil {
-		return fmt.Errorf("unknown shell %q", shellName)
+	w.IfGitVersionIsAtLeast(gitMinVersionCloneWithRef)
+	defer w.EndIf()
+
+	if err := b.writeCloneRevisionCmd(w, info, templateDir, credConfigFile); err != nil {
+		return err
 	}
 
-	remoteHost, err := b.getRemoteHost(build)
-	if err != nil {
-		return fmt.Errorf("getting remote host: %w", err)
+	w.Else()
+
+	if err := b.writeRefspecFetchCmd(w, info, templateDir, credConfigFile); err != nil {
+		return err
 	}
 
-	// Scenario: shell executor & git-credential-manager aka. GCM
-	//
-	// When GCM is at play, and it's configured in the system scope (default for wingit), then it might cache the
-	// credentials from the last pipeline run, ie. the CI_JOB_TOKEN. This token is not valid anymore. Same might be true
-	// for other globally installed credential helpers.
-	//
-	// Thus we clear the cache here.
-	//
-	// This is, strictly speaking, only needed if some git credential helper caches creds across runs, but should not do
-	// harm if nothing is caching creds or the cache is empty.
-	//
-	// `git credential reject` will call all credential helpers with "erase", and therefor prune the cache for this URL
-	// everywhere.
-	w.CommandWithStdin("url="+remoteHost, "git", "credential", "reject")
-
-	credHelperCommand := "!" + shell.GetGitCredHelperCommand(info.RuntimeOS)
-	credSection := "credential." + remoteHost
-
-	w.Command("git", "config", "-f", credConfigFile, credSection+".username", "gitlab-ci-token")
-	w.Command("git", "config", "-f", credConfigFile, credSection+".helper", credHelperCommand)
-
-	w.Command("git", "config", "include.path", credConfigFile)
-
 	return nil
 }
 
+func (b *AbstractShell) writeCloneRevisionCmd(w ShellWriter, info common.ShellScriptInfo, templateDir, credConfigFile string) error {
+	build := info.Build
+	projectDir := build.FullProjectDir()
+	depth := build.GitInfo.Depth
+
+	switch {
+	case depth > 0:
+		w.Noticef("Cloning repository for %s with git depth set to %d...", build.GitInfo.Ref, depth)
+	case build.GitInfo.Ref != "":
+		w.Noticef("Cloning repository for %s...", build.GitInfo.Ref)
+	default:
+		w.Noticef("Cloning repository...")
+	}
+
+	remoteURL, err := build.GetRemoteURL()
+	if err != nil {
+		return fmt.Errorf("writing clone command: %w", err)
+	}
+
+	v := common.AppVersion
+	userAgent := fmt.Sprintf("http.userAgent=%s %s %s/%s", v.Name, v.Version, v.OS, v.Architecture)
+
+	cloneArgs := []string{"-c", userAgent, "clone", "--no-checkout", remoteURL, projectDir, "--template", templateDir}
+
+	if depth > 0 {
+		cloneArgs = append(cloneArgs, "--depth", strconv.Itoa(depth))
+	}
+
+	if strings.HasPrefix(build.GitInfo.Ref, "refs/") {
+		cloneArgs = append(cloneArgs, "--revision", build.GitInfo.Ref)
+	} else if build.GitInfo.Ref != "" {
+		cloneArgs = append(cloneArgs, "--branch", build.GitInfo.Ref)
+	}
+
+	if credConfigFile != "" {
+		// we don't have a git repo / config yet, thus adding cred helper explicitly as an arg.
+		cloneArgs = append(cloneArgs, "-c", "include.path="+credConfigFile)
+	}
+
+	w.Command("git", cloneArgs...)
+	w.Cd(projectDir)
+
+	if credConfigFile != "" {
+		// now that we have a git repo / config, we persist the cred helper.
+		w.Command("git", "config", "include.path", credConfigFile)
+	}
+
+	return nil
+}
+
+func (b *AbstractShell) setupTemplateDir(w ShellWriter, build *common.Build, projectDir string) string {
+	templateDir := w.MkTmpDir(gitTemplateDir)
+	templateFile := w.Join(templateDir, "config")
+
+	if build.SafeDirectoryCheckout {
+		// Solves problem with newer Git versions when files existing in the working directory
+		// are owned by different system owners. This may happen for example with Docker executor,
+		// a root-less image used in previous job and the working directory being persisted between
+		// jobs. More details can be found at https://gitlab.com/gitlab-org/gitlab/-/issues/368133.
+		w.Command("git", "config", "--global", "--add", "safe.directory", projectDir)
+	}
+
+	w.Command("git", "config", "-f", templateFile, "init.defaultBranch", "none")
+	w.Command("git", "config", "-f", templateFile, "fetch.recurseSubmodules", "false")
+	w.Command("git", "config", "-f", templateFile, "credential.interactive", "never")
+	w.Command("git", "config", "-f", templateFile, "gc.autoDetach", "false")
+
+	if build.IsFeatureFlagOn(featureflags.UseGitBundleURIs) {
+		w.Command("git", "config", "-f", templateFile, "transfer.bundleURI", "true")
+	}
+
+	if build.IsSharedEnv() {
+		b.writeGitSSLConfig(w, build, []string{"-f", templateFile})
+	}
+
+	return templateDir
+}
+
 func (b *AbstractShell) writeGitCleanup(w ShellWriter, build *common.Build) {
 	projectDir := build.FullProjectDir()
 	submoduleStrategy := build.GetSubmoduleStrategy()
@@ -1394,6 +1478,8 @@ func (b *AbstractShell) writeCleanupScript(_ context.Context, w ShellWriter, inf
 
 	b.writeGitCleanup(w, info.Build)
 
+	w.RmFile(w.Join(info.Build.RootDir, BuildUidGidFile))
+
 	return nil
 }
 
